/*
* JBoss, Home of Professional Open Source.
* Copyright 2006, Red Hat Middleware LLC, and individual contributors
* as indicated by the @author tags. See the copyright.txt file in the
* distribution for a full listing of individual contributors. 
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/ 
package org.jboss.reflect.plugins.bytecode.bytes;

import java.lang.annotation.Annotation;

/**
 * Base interface containing bytecode information for a class or a member
 * 
 * @author <a href="kabir.khan@jboss.com">Kabir Khan</a>
 * @version $Revision: 1.1 $
 */
public interface Bytes
{
   /**
    * Gets the modifiers for this class or member
    * 
    * @return the modifiers
    */
   int getModifiers();
   
   
   /**
    * Gets the generic signature of this class or member as contained
    * in the Signature attribute
    * 
    * @return the generic signature, or null if there is no signature
    */
   String getGenericSignature();
   
   /**
    * Gets the annotations on this class or member
    * 
    * @return the annotations, or null if there are none
    */
   Annotation[] getAnnotations();
}
